package com.gyee.redisread.task;

import com.gyee.feign.api.Redis2TaosApi;
import com.gyee.redisread.config.RedisConfig;
import com.gyee.redisread.util.ChangedSave;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.Pipeline;

import java.net.URI;
import java.util.List;
import java.util.Objects;

public class TaskRedisRead implements Runnable {
    private final List<String> keys;
    private RedisConfig redisConfig;
    private String dataType;
    private Redis2TaosApi redis2TaosApi;

    public static final Logger log = LoggerFactory.getLogger(TaskRedisRead.class);

    public TaskRedisRead(List<String> keys, RedisConfig redisConfig, String dataType, Redis2TaosApi redis2TaosApi) {
        this.keys = keys;
        this.redisConfig = redisConfig;
        this.dataType = dataType;
        this.redis2TaosApi = redis2TaosApi;
    }

    @Override
    public void run() {

        List<Object> objects;

        String pointid = null;

        try (Jedis jedis = redisConfig.getJedis(); Pipeline pipelined = jedis.pipelined();) {

            for (String key : keys) {
                //pipelined.hscan(key, "0");
                pipelined.hmget(key, "pointid", "value", "timestamp");
            }
            //redis批量获取接收为Object
            objects = pipelined.syncAndReturnAll();

            StringBuilder sb = new StringBuilder();
            sb.append("INSERT INTO ");

            String value;
            String datetime;
            double val;
            long ts;
            boolean bl;
            List<String> response;
            String sb2str;

            if ("bool".equals(dataType)) {
                for (int i = 0; i < objects.size(); i++) {
                    try {
                        response = (List<String>) objects.get(i);
                    }catch (Exception e){
                        log.error(objects.get(i).toString());
                        continue;
                    }

                    value = response.get(1);
                    if (value == null || "".equals(value) || value.contains("-nan")) continue;
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if (ChangedSave.map.containsKey(pointid)) {
                        if (!Objects.equals(ChangedSave.map.get(pointid), datetime)) {
                            ChangedSave.map.put(pointid, datetime);
                            ts = Long.valueOf(datetime) * 1000;
                            bl = Double.valueOf(value) == 1;
                            //j++;
                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(bl).append(") ");
                        }
                    } else {
                        ChangedSave.map.put(pointid, datetime);
                    }

                    /*ts = Long.valueOf(datetime) * 1000;
                    bl = Double.valueOf(value) == 1;
                    sb.append(pointid).append(" VALUES (").append(ts).append(",").append(bl).append(") ");*/
                }
                sb2str = sb.toString();
                if ("INSERT INTO ".equals(sb2str)) return;
                for (String taosUrl : redisConfig.getTaosUrls().split(",")) {
                    redis2TaosApi.putSql(URI.create(taosUrl), sb2str);
                }
            } else {
                for (int i = 0; i < objects.size(); i++) {
                    try {
                        response = (List<String>) objects.get(i);
                    }catch (Exception e){
                        log.error(objects.get(i).toString());
                        continue;
                    }

                    value = response.get(1);
                    if (value == null || "".equals(value) || value.contains("-nan")) continue;
                    datetime = response.get(2);
                    if (StringUtils.isEmpty(datetime)) continue;
                    pointid = response.get(0);
                    if (StringUtils.isEmpty(pointid)) continue;

                    if (ChangedSave.map.containsKey(pointid)) {
                        if (!Objects.equals(ChangedSave.map.get(pointid), datetime)) {
                            ChangedSave.map.put(pointid, datetime);
                            ts = Long.valueOf(datetime) * 1000;
                            val = Double.valueOf(value);

                            //j++;
                            sb.append(pointid).append(" VALUES (").append(ts).append(",").append(val).append(") ");
                        }
                    } else {
                        ChangedSave.map.put(pointid, datetime);
                    }
                    /*ts = Long.valueOf(datetime) * 1000;
                    val = Double.valueOf(value);
                    sb.append(pointid).append(" VALUES (").append(ts).append(",").append(val).append(") ");*/
                }
                sb2str = sb.toString();
                if ("INSERT INTO ".equals(sb2str)) return;
                for (String taosUrl : redisConfig.getTaosUrls().split(",")) {
                    redis2TaosApi.putSql(URI.create(taosUrl), sb2str);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(pointid + "------写请求超时或读取redis出错！");
        }
        System.out.print("*");
    }
}
